import { Signal } from "@hmans/signal";
import { RegisteredEntity } from ".";
import { EntityWith, IEntity } from "./World";
/**
 * A query is an array of component names.
 */
export type Query<T extends IEntity> = (keyof T)[];
export type ArchetypeEntity<E extends IEntity, Q extends Query<E> = Query<E>> = EntityWith<RegisteredEntity<E>, Q[number]>;
export declare class Archetype<E extends IEntity, Q extends Query<E> = Query<E>> {
    query: Q;
    /** A list of entities belonging to this archetype. */
    entities: ArchetypeEntity<E, Q>[];
    constructor(query: Q);
    [Symbol.iterator](): IterableIterator<ArchetypeEntity<E, Q>>;
    /** Listeners on this event are invoked when an entity is added to this archetype's index. */
    onEntityAdded: Signal<ArchetypeEntity<E, Q>, (payload: ArchetypeEntity<E, Q>) => void>;
    /** Listeners on this event are invoked when an entity is removed from this archetype's index. */
    onEntityRemoved: Signal<RegisteredEntity<E>, (payload: RegisteredEntity<E>) => void>;
    indexEntity(entity: RegisteredEntity<E>): void;
    removeEntity(entity: RegisteredEntity<E>): void;
}
